﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Text;
using System.IO;
using System.Linq;
using System.Security.Cryptography.Xml;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace PuzzlePlayer_Namespace
{
    internal class Skyscrapers : Board
    {
        public int[,] visionState;
        public string[,] candidateState;
        public Skyscrapers(int boardSize = 6)
        {
            boardState = GetClearBoard(boardSize + 2);
            lastGeneratedBoard = GetClearBoard(boardSize + 2);
            candidateState = new string[boardSize + 2, boardSize + 2];
            description = "lol";
            drawFactor = 1;
            candidateState[1, 1] = "123";
            candidateState[2, 3] = "1734";
            candidateState[4, 4] = "123456789";
            boardState[1, 1] = 1;
            boardState[2, 2] = 2;
            boardState[3, 3] = 3;
            boardState[4, 4] = 4;
        }
        public override void Draw(Graphics gr, Rectangle r)
        {
            Size tilesize = new Size(r.Width / boardState.GetLength(0), r.Height / boardState.GetLength(1));
            int stringsize = Math.Min(tilesize.Height / 2, tilesize.Width);
            StringFormat stringFormat = new StringFormat();
            stringFormat.LineAlignment = StringAlignment.Center;
            stringFormat.Alignment = StringAlignment.Center;
            gr.FillRectangle(Brushes.Beige, r.X + tilesize.Width, r.Y + tilesize.Height, r.Width - 2 * tilesize.Width, r.Height - 2 * tilesize.Height);
            for (int i = 0; i < boardState.GetLength(0); i++)
            {
                for (int j = 0; j < boardState.GetLength(1); j++)
                {
                    if (i == 0 || i == boardState.GetLength(0) - 1 || j == 0 || j == boardState.GetLength(1) - 1)
                    {
                        if (boardState[i, j] != emptySpace)
                        {
                            gr.DrawString(Convert.ToString(boardState[i, j]),
                            new Font("Verdana", stringsize / 2),
                            Brushes.Black,
                            (int)(r.X + ((double)i + 0.5) * tilesize.Width),
                            (int)(r.Y + ((double)j + 0.5) * tilesize.Height),
                            stringFormat);
                        }
                    }
                    else
                    {
                        gr.DrawRectangle(Pens.DarkGray,
                        r.X + i * tilesize.Width,
                        r.Y + j * tilesize.Height,
                        tilesize.Width,
                        tilesize.Height);
                        if (boardState[i, j] != emptySpace)
                        {
                            gr.DrawString(Convert.ToString(boardState[i, j]),
                            new Font("Verdana", stringsize),
                            Brushes.Black,
                            (int)(r.X + ((double)i + 0.5) * tilesize.Width),
                            (int)(r.Y + ((double)j + 0.5) * tilesize.Height),
                            stringFormat);
                        }
                        else 
                        {
                            if (candidateState[i, j] == null) continue;
                            foreach (char c in candidateState[i, j])
                            {
                                gr.DrawString(Convert.ToString(c),
                                new Font("Verdana", stringsize / 3),
                                Brushes.Black,
                                (int)(r.X + ((double)i + 0.25 * ((char.GetNumericValue(c) - 1) % 3 + 1)) * tilesize.Width),
                                (int)(r.Y + ((double)j + 0.25 * Math.Ceiling(char.GetNumericValue(c) / 3)) * tilesize.Height),
                                stringFormat);
                            }
                        }
                    }
                }
            }
        }

        public override SOLUTIONS Solve(bool CheckOnly)
        {
            return SOLUTIONS.NONE;
        }

        public override void Generate()
        {
            boardState = GetClearBoard(boardState.GetLength(0));
            candidateState = new string[boardState.GetLength(0), boardState.GetLength(1)];
            Random random = new Random();
            void RandomFillLine(int direction)
            {
                List<int> options = new List<int>();
                for (int i = 0; i < boardState.GetLength(0) - 2; i++)
                    options.Add(i + 1);
                int deficit = 0;
                if (boardState[1, 1] != emptySpace)
                {
                    options.Remove(boardState[1, 1]);
                    deficit++;
                }
                for (int i = deficit; i < boardState.GetLength(0) - 2; i++)
                {
                    int index = random.Next(options.Count);
                    boardState[1 + i * (1 - direction), 1 + i * direction] = options[index];
                    options.RemoveAt(index);
                }
            }
            RandomFillLine(0);
            RandomFillLine(1);
            return;
        }

        public override void TileInput(Point? p, Keys k)
        {
            if (p == null) return;
            double center = ((double)boardState.GetLength(0) - 1) / 2;
            if (Math.Abs(((Point)p).X - center) == center && Math.Abs(((Point)p).Y - center) == center) return;
            int num = (int)k - 48;
            if (num > 0 && num <= boardState.GetLength(0) - 2) boardState[((Point)p).X, ((Point)p).Y] = num;
        }

        protected override List<Move> GetSolveList(int[,] boardToSolve)
        {
            return new List<Move>();
        }
    }
}
